@@ -17,7 +17,7 @@ namespace from_json {
     R_xlen_t vec_length = x_size* n;
     
     // each list element MUST be length n
-    for( i = 0; i < x_size; i++ ) {
+    for( i = 0; i < x_size; ++i ) {
       if( Rf_length( x[i] ) != n ) {
         Rcpp::stop("jsonify - list elements different sizes");
       }
@@ -80,14 +80,14 @@ namespace from_json {
     // them in an R vector
     
     int r_type = 0;
-    int first_r_type; // for keeping track if the vector has been coerced when simplified
+    // int first_r_type; // for keeping track if the vector has been coerced when simplified
     // every member of the array will be coerced to the max d-type sexp type
     // but first, get each element and put into a list
     R_xlen_t arr_len = array.Size();
     R_xlen_t i;
     Rcpp::List out( arr_len );
     
-    for( i = 0; i < arr_len; i++ ) {
+    for( i = 0; i < arr_len; ++i ) {
       
       switch(array[i].GetType()) {
       // bool - false
@@ -106,7 +106,7 @@ namespace from_json {
         
         // string
       case rapidjson::kStringType: {
-        out[i] = array[i].GetString();
+        out[i] = Rcpp::String(array[i].GetString());
         r_type = STRSXP > r_type ? STRSXP : r_type;
         break;
       }
@@ -136,9 +136,9 @@ namespace from_json {
         Rcpp::stop("jsonify - array_to_vector only able to parse int, double, string and bool");
       }
       }
-      if( i == 0 ) {
-        first_r_type = r_type;
-      }
+      // if( i == 0 ) {
+      //   first_r_type = r_type;
+      // }
     }
     
     if( simplify ) {
@@ -149,18 +149,20 @@ namespace from_json {
 
   inline SEXP simplify_matrix(
       Rcpp::List& out,
-      int& n_col, 
-      int& n_row,
+      R_xlen_t& n_col, 
+      R_xlen_t& n_row,
       int& r_type
   ) {
     
+    R_xlen_t i, j;
+    
     switch( r_type ) {
     case INTSXP: {
       
       // if( by == "col") {
       //   Rcpp::IntegerMatrix mat( n_col, n_row ); // swapped
       // 
-      //   for(int i = 0; i < n_col; i++ ) {
+      //   for(int i = 0; i < n_col; ++i ) {
       //     Rcpp::IntegerVector this_vec = out[i];
       //     mat( Rcpp::_, i ) = this_vec;
       //   }
@@ -168,9 +170,10 @@ namespace from_json {
       // } else {
       
       Rcpp::IntegerMatrix mat( n_row, n_col );
-      for( int i = 0; i < n_row; i++ ) {
+      
+      for( i = 0; i < n_row; ++i ) {
         Rcpp::IntegerVector this_vec = out[i];
-        for( int j = 0; j < n_col; j++ ) {
+        for( j = 0; j < n_col; ++j ) {
           int this_val = this_vec[j];
           mat( i, j ) = this_val;
         }
@@ -182,7 +185,7 @@ namespace from_json {
       // if( by == "col" ) {
       //   Rcpp::NumericMatrix mat( n_row, n_col );
       //   //if by == col
-      //   for(int i = 0; i < n_col; i++ ) {
+      //   for(int i = 0; i < n_col; ++i ) {
       //     Rcpp::NumericVector this_vec = out[i];
       //     mat( Rcpp::_, i ) = this_vec;
       //   }
@@ -190,9 +193,9 @@ namespace from_json {
       // } else {
       
       Rcpp::NumericMatrix mat( n_row, n_col );
-      for( int i = 0; i < n_row; i++ ) {
+      for( i = 0; i < n_row; ++i ) {
         Rcpp::NumericVector this_vec = out[i];
-        for( int j = 0; j < n_col; j++ ) {
+        for( j = 0; j < n_col; ++j ) {
           double this_val = this_vec[j];
           mat( i, j ) = this_val;
         }
@@ -204,7 +207,7 @@ namespace from_json {
       // if( by == "col" ) {
       //   Rcpp::LogicalMatrix mat( n_row, n_col );
       //   //if by == col
-      //   for(int i = 0; i < n_col; i++ ) {
+      //   for(int i = 0; i < n_col; ++i ) {
       //     Rcpp::LogicalVector this_vec = out[i];
       //     mat( Rcpp::_, i ) = this_vec;
       //   }
@@ -212,9 +215,9 @@ namespace from_json {
       // } else {
       
       Rcpp::LogicalMatrix mat( n_row, n_col );
-      for( int i = 0; i < n_row; i++ ) {
+      for( i = 0; i < n_row; ++i ) {
         Rcpp::LogicalVector this_vec = out[i];
-        for( int j = 0; j < n_col; j++ ) {
+        for( j = 0; j < n_col; ++j ) {
           bool this_val = this_vec[j];
           mat( i, j ) = this_val;
         }
@@ -231,16 +234,16 @@ namespace from_json {
       // if( by == "col" ) {
       //   Rcpp::StringMatrix mat( n_row, n_col );
       //   //if by == col
-      //   for(int i = 0; i < n_col; i++ ) {
+      //   for(int i = 0; i < n_col; ++i ) {
       //     Rcpp::StringVector this_vec = out[i];
       //     mat( Rcpp::_, i ) = this_vec;
       //   }
       //   return mat;
       // } else {
       Rcpp::StringMatrix mat( n_row, n_col );
-      for( int i = 0; i < n_row; i++ ) {
+      for( i = 0; i < n_row; ++i ) {
         Rcpp::StringVector this_vec = out[i];
-        for( int j = 0; j < n_col; j++ ) {
+        for( j = 0; j < n_col; ++j ) {
           Rcpp::String this_val = this_vec[j];
           mat( i, j ) = this_val;
         }
@@ -257,10 +260,11 @@ namespace from_json {
   ) {
     R_xlen_t n = array_of_array.size();
     R_xlen_t j;
-    std::unordered_set<int> array_lengths;
-    std::unordered_set<int> array_types;
+    std::unordered_set< R_xlen_t > array_lengths;
+    std::unordered_set< int > array_types;
     bool can_be_matrix = true;
-    for( j = 0; j < n; j++ ) {
+    
+    for( j = 0; j < n; ++j ) {
       SEXP s = array_of_array[j];
       int this_type = TYPEOF( s );
       if( Rf_isMatrix( s ) || this_type == VECSXP ) {
@@ -281,8 +285,9 @@ namespace from_json {
     if( can_be_matrix ) {
       Rcpp::IntegerVector arr_types( array_types.begin(), array_types.end() );
       int r_type = Rcpp::max( arr_types );
-      int n_col = *array_lengths.begin();  // only one sizez
-      int n_row = n;
+      R_xlen_t n_col = *array_lengths.begin();  // only one sizez
+      R_xlen_t n_row = n;
+
       return jsonify::from_json::simplify_matrix( array_of_array, n_col, n_row, r_type );
     } else {
       return array_of_array;
@@ -294,8 +299,8 @@ namespace from_json {
   inline void list_to_vector(
       Rcpp::List& columns,
       std::string& this_name,
-      int& r_type,
-      int& struct_type, // 1 vector, 2 matrix, 3 list
+      R_xlen_t& r_type,
+      R_xlen_t& struct_type, // 1 vector, 2 matrix, 3 list
       bool fill_na
   ) {
     
@@ -317,7 +322,7 @@ namespace from_json {
         switch( r_type ) {
         case LGLSXP: {
           Rcpp::LogicalMatrix m( n_rows, n_cols );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             Rcpp::LogicalVector v = Rcpp::as< Rcpp::LogicalVector >( lst[i] );
             m( i, Rcpp::_ ) = v;
           }
@@ -326,7 +331,7 @@ namespace from_json {
         }
         case INTSXP: {
           Rcpp::IntegerMatrix m( n_rows, n_cols );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             Rcpp::IntegerVector v = Rcpp::as< Rcpp::IntegerVector >( lst[i] );
             m( i, Rcpp::_ ) = v;
           }
@@ -335,7 +340,7 @@ namespace from_json {
         }
         case REALSXP: {
           Rcpp::NumericMatrix m( n_rows, n_cols );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             Rcpp::NumericVector v = Rcpp::as< Rcpp::NumericVector >( lst[i] );
             m( i, Rcpp::_ ) = v;
           }
@@ -344,7 +349,7 @@ namespace from_json {
         }
         case STRSXP: {
           Rcpp::StringMatrix m( n_rows, n_cols );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             Rcpp::StringVector v = Rcpp::as< Rcpp::StringVector >( lst[i] );
             m( i, Rcpp::_ ) = v;
           }
@@ -367,7 +372,7 @@ namespace from_json {
         switch( r_type ) {
         case LGLSXP: {
           Rcpp::LogicalVector lv( n_rows );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             if( Rf_isNull( lst[i] ) ) {
               lv[i] = NA_LOGICAL;
             } else {
@@ -379,7 +384,7 @@ namespace from_json {
         }
         case INTSXP: {
           Rcpp::IntegerVector iv( n_rows );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             if( Rf_isNull( lst[i] ) ) {
               iv[i] = NA_INTEGER;
             } else {
@@ -391,7 +396,7 @@ namespace from_json {
         }
         case REALSXP: {
           Rcpp::NumericVector nv( n_rows );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             if( Rf_isNull( lst[i] ) ) {
               nv[i] = NA_REAL;
             } else {
@@ -403,7 +408,7 @@ namespace from_json {
         }
         case STRSXP: {
           Rcpp::StringVector sv( n_rows );
-          for( i = 0; i < n_rows; i++ ) {
+          for( i = 0; i < n_rows; ++i ) {
             if( Rf_isNull( lst[i] ) ) {
               sv[i] = NA_STRING;
             } else {
@@ -437,7 +442,7 @@ namespace from_json {
 
   inline SEXP simplify_dataframe_fill_na(
       Rcpp::List& out,
-      int& doc_len
+      R_xlen_t& doc_len
   ) {
     
     // the number of rows is equal to the number of list elements?
@@ -449,22 +454,22 @@ namespace from_json {
     // keep track of it's type
     Rcpp::List columns;        // will grow when a new column name is found
     
-    std::unordered_map< std::string, int > column_types;
-    std::unordered_map< std::string, int > column_structs; // int : 1 == vector element, 2 == matrix, 3 == list;
-    std::unordered_map< std::string, int > column_lengths;
+    std::unordered_map< std::string, R_xlen_t > column_types;
+    std::unordered_map< std::string, R_xlen_t > column_structs; // int : 1 == vector element, 2 == matrix, 3 == list;
+    std::unordered_map< std::string, R_xlen_t > column_lengths;
     
-    int struct_type;
-    int sexp_length;
-    int tp;
-    int st;
-    int ln;
+    R_xlen_t struct_type;
+    R_xlen_t sexp_length;
+    R_xlen_t tp;
+    R_xlen_t st;
+    R_xlen_t ln;
     
     Rcpp::StringVector list_names;
     std::vector< std::string > column_names;
     bool new_column = true;
     
     
-    for( i = 0; i < n_rows; i++ ) {
+    for( i = 0; i < n_rows; ++i ) {
       // iterating list elements
       Rcpp::List this_list = out[i];
 
@@ -479,7 +484,7 @@ namespace from_json {
       }
       
       // Iterate over names??
-      for( j = 0; j < list_size; j++ ) { 
+      for( j = 0; j < list_size; ++j ) { 
         const char* this_name = list_names[j];
             
         // does this_name exist in the vector of names?
@@ -547,7 +552,7 @@ namespace from_json {
     for( auto& it: column_types ) {
       
       std::string this_name = it.first;
-      int r_type = it.second;
+      R_xlen_t r_type = it.second;
       struct_type = column_value( column_structs, this_name.c_str() );
       if( struct_type == 3 ) {
         // can it be a data.frame?
@@ -559,8 +564,11 @@ namespace from_json {
     }
     
     columns.attr("class") = "data.frame";
-    columns.attr("row.names") = Rcpp::seq(1, n_rows);
-    
+    if( n_rows > 0 ) {
+      columns.attr("row.names") = Rcpp::seq(1, n_rows);
+    } else {
+      columns.attr("row.names") = Rcpp::IntegerVector(0);
+    }
     return columns;
   }
 
@@ -569,7 +577,7 @@ namespace from_json {
   // iff any column lenghts are different, it's a list
   inline SEXP simplify_dataframe(
       Rcpp::List& out,
-      int& doc_len
+      R_xlen_t& doc_len
   ) {
     
     // the number of rows is equal to the number of list elements?
@@ -581,19 +589,19 @@ namespace from_json {
     // keep track of it's type
     Rcpp::List columns;        // will grow when a new column name is found
     
-    std::unordered_map< std::string, int > column_types;
-    std::unordered_map< std::string, int > column_structs; // int : 1 == vector element, 2 == matrix, 3 == list;
-    std::unordered_map< std::string, int > column_lengths;
+    std::unordered_map< std::string, R_xlen_t > column_types;
+    std::unordered_map< std::string, R_xlen_t > column_structs; // int : 1 == vector element, 2 == matrix, 3 == list;
+    std::unordered_map< std::string, R_xlen_t > column_lengths;
     
-    int struct_type;
-    int sexp_length;
-    int tp;
-    int st;
-    int ln;
+    R_xlen_t struct_type;
+    R_xlen_t sexp_length;
+    R_xlen_t tp;
+    R_xlen_t st;
+    R_xlen_t ln;
     
     Rcpp::StringVector list_names;
     
-    for( i = 0; i < n_rows; i++ ) {
+    for( i = 0; i < n_rows; ++i ) {
       // iterating list elements
       Rcpp::List this_list = out[i];
       if( i == 0 ) {
@@ -608,10 +616,10 @@ namespace from_json {
       }
       
       // Iterate over names??
-      for( j = 0; j < list_size; j++ ) { 
+      for( j = 0; j < list_size; ++j ) { 
         const char* this_name = list_names[j];
         Rcpp::StringVector these_names = this_list.names();
-        int found_name = where_is( this_name, these_names );
+        R_xlen_t found_name = where_is( this_name, these_names );
         
         if( found_name == -1 ) {
           // can't simplify
@@ -680,7 +688,7 @@ namespace from_json {
     for( auto& it: column_types ) {
       
       std::string this_name = it.first;
-      int r_type = it.second;
+      R_xlen_t r_type = it.second;
       struct_type = column_value( column_structs, this_name.c_str() );
       if( struct_type == 3 ) {
         // can it be a data.frame?
@@ -693,7 +701,11 @@ namespace from_json {
     
     //columns.attr("names") = names;
     columns.attr("class") = "data.frame";
-    columns.attr("row.names") = Rcpp::seq(1, n_rows);
+    if( n_rows > 0 ) {
+      columns.attr("row.names") = Rcpp::seq(1, n_rows);
+    } else {
+      columns.attr("row.names") = Rcpp::IntegerVector(0);
+    }
     
     return columns;
   }
